package org.mercury_im.messenger.ui.account;

import android.app.Application;

import androidx.lifecycle.AndroidViewModel;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;

import org.mercury_im.messenger.MercuryImApplication;
import org.mercury_im.messenger.Messenger;
import org.mercury_im.messenger.data.repository.AccountRepository;
import org.mercury_im.messenger.entity.Account;
import org.mercury_im.messenger.logging.Tags;
import org.mercury_im.messenger.util.DebugUtils;
import org.mercury_im.messenger.xmpp.MercuryConnection;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.inject.Inject;

import io.reactivex.disposables.CompositeDisposable;

public class AccountsViewModel extends AndroidViewModel {

    private static final Logger LOGGER = Logger.getLogger(Tags.TAG_ANDROID);

    @Inject
    AccountRepository repository;

    @Inject
    Messenger messenger;

    private final MutableLiveData<List<MercuryConnection>> connections = new MutableLiveData<>();
    private final CompositeDisposable compositeDisposable = new CompositeDisposable();

    @Inject
    public AccountsViewModel(Application application) {
        super(application);
        MercuryImApplication.getApplication().getAppComponent().inject(this);
        compositeDisposable.add(repository.observeAllAccounts()
        .map(accounts -> {
            List<MercuryConnection> connections = new ArrayList<>(accounts.size());
            for (Account account : accounts) {
                connections.add(messenger.getConnectionManager().getOrCreateConnection(account));
            }
            return connections;
        }).subscribe(connections::postValue));
    }

    @Override
    protected void onCleared() {
        super.onCleared();
        LOGGER.log(Level.INFO, "AccountsViewModel.onCleared()");
        compositeDisposable.clear();
    }

    public CompositeDisposable getDisposable() {
        return compositeDisposable;
    }

    public LiveData<List<MercuryConnection>> getConnections() {
        return connections;
    }

    public void setAccountEnabled(Account accountModel, boolean enabled) {
        accountModel.setEnabled(enabled);
        compositeDisposable.add(repository.upsertAccount(accountModel)
                .subscribe());
    }
}
